// Copyright (c) 2022-2023 VMware Carbon Black, all rights reserved

package hinttype

import (
	"encoding/json"
	"log"
	"testing"
)

func TestHintComparison(t *testing.T) {
	firstHints := HintSet(0)
	secondHints := HintSet(10)
	thirdHints := HintSet(0)
	if CompareHintSets(&firstHints, &secondHints) {
		t.Fatalf("Hint sets are different but comparison result indicates equality")
	}
	if !CompareHintSets(&firstHints, &thirdHints) {
		t.Fatalf("Hint sets are the same, but comparison result indicates inequality")
	}
}

func TestHintValue(t *testing.T) {
	// NOTE: More of a useful utility than a test -- just change hintValue and execute the test to get hints
	hintValue := 1690552024753
	hints := HintSet(hintValue)
	hintArray := hints.ToStringArray()

	log.Printf("Value %d -> %v", hintValue, hintArray)
}

func TestHintSetToArray(t *testing.T) {
	hints := HintSet(42)
	check := hints.ToIntArray()

	log.Printf("hint array %v", check)
	if len(check) != 3 {
		t.Fatalf("incorrect hint count!")
	}
	if check[0] != 1 || check[1] != 3 || check[2] != 5 {
		t.Fatalf("incorrect conversion to int array!")
	}
}

func TestHintSetToArrayWithRep(t *testing.T) {
	hints := NewHintSet(REP_REQUEST)
	check := hints.ToIntArray()
	if len(check) != 1 {
		t.Fatalf("incorrect hint count")
	}
	if check[0] != 32 {
		t.Fatalf("didn't find 32nd hint")
	}
}

func TestHintType(t *testing.T) {
	testHintSet := NewHintSet(NEW_POLICY, HASH_LIST_REFRESH, REREGISTER, YARA_PE_HEADER_SIGNATURES, REP_REQUEST)

	if !testHintSet.IsHintSet(NEW_POLICY) {
		t.Fatalf("Expected new policy to be set!")
	}

	if testHintSet.IsHintSet(NEW_CONFIG_DATA) {
		t.Fatalf("Expected new config data to be false")
	}

	if !testHintSet.IsHintSet(HASH_LIST_REFRESH) {
		t.Fatalf("Expected hash list refresh to be set")
	}

	bytes, err := json.Marshal(&testHintSet)
	if err != nil {
		t.Fatalf("Expected hintset to marshal OK")
	}

	var anotherTestHintSet HintSet
	err = json.Unmarshal(bytes, &anotherTestHintSet)
	if err != nil {
		t.Fatalf("Hintset did not unmarshal")
	}

	if !anotherTestHintSet.IsHintSet(NEW_POLICY) {
		t.Fatalf("Expected new policy to be set!")
	}

	if anotherTestHintSet.IsHintSet(NEW_CONFIG_DATA) {
		t.Fatalf("Expected new config data to be false")
	}

	if !anotherTestHintSet.IsHintSet(HASH_LIST_REFRESH) {
		t.Fatalf("Expected hash list refresh to be set")
	}

	if anotherTestHintSet.IsHintSet(BLADE_ACTION) {
		t.Fatalf("Found unexpected blade action hint")
	}

	if !anotherTestHintSet.IsHintSet(REREGISTER) {
		t.Fatalf("Failed to find reregister hint")
	}

	if !anotherTestHintSet.IsHintSet(YARA_PE_HEADER_SIGNATURES) {
		t.Fatalf("Failed to find yara pe signatures hint")
	}

	if !testHintSet.IsHintSet(REP_REQUEST) {
		t.Fatalf("Failed to find rep request hint")
	}
}
